@@ -6,7 +6,7 @@
  * You may not use this file except in compliance with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -28,13 +28,16 @@
 #if defined(__x86)
 #include <linux/simd_x86.h>
 
+#elif defined(__arm__)
+#include <linux/simd_arm.h>
+
 #elif defined(__aarch64__)
 #include <linux/simd_aarch64.h>
 
 #elif defined(__powerpc__)
 #include <linux/simd_powerpc.h>
-#else
 
+#else
 #define	kfpu_allowed()		0
 #define	kfpu_begin()		do {} while (0)
 #define	kfpu_end()		do {} while (0)
