@@ -6,7 +6,7 @@
  * You may not use this file except in compliance with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -30,6 +30,7 @@
 #include <sys/wmsum.h>
 #include <sys/dmu.h>
 #include <sys/kstat.h>
+#include <sys/zil.h>
 
 typedef struct dataset_sum_stats_t {
 	wmsum_t dss_writes;
@@ -56,14 +57,19 @@ typedef struct dataset_kstat_values {
 	 * entry is removed from the unlinked set
 	 */
 	kstat_named_t dkv_nunlinked;
+	/*
+	 * Per dataset zil kstats
+	 */
+	zil_kstat_values_t dkv_zil_stats;
 } dataset_kstat_values_t;
 
 typedef struct dataset_kstats {
 	dataset_sum_stats_t dk_sums;
+	zil_sums_t dk_zil_sums;
 	kstat_t *dk_kstats;
 } dataset_kstats_t;
 
-void dataset_kstats_create(dataset_kstats_t *, objset_t *);
+int dataset_kstats_create(dataset_kstats_t *, objset_t *);
 void dataset_kstats_destroy(dataset_kstats_t *);
 
 void dataset_kstats_update_write_kstats(dataset_kstats_t *, int64_t);
