@@ -29,6 +29,6 @@
 #ifndef ZFS_CONTEXT_OS_H_
 #define	ZFS_CONTEXT_OS_H_
 
-#define	ZFS_EXPORTS_PATH	"/etc/zfs/exports"
+#define	HAVE_LARGE_STACKS	1
 
 #endif
