@@ -7,7 +7,7 @@
  * with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -41,7 +41,7 @@
 
 #define	BUFSIZE	(MNT_LINE_MAX + 2)
 
-__thread char buf[BUFSIZE];
+static __thread char buf[BUFSIZE];
 
 #define	DIFF(xx)	( \
 	    (mrefp->xx != NULL) && \
@@ -83,7 +83,7 @@ _sol_getmntent(FILE *fp, struct mnttab *mgetp)
 }
 
 static int
-getextmntent_impl(FILE *fp, struct extmnttab *mp, int len)
+getextmntent_impl(FILE *fp, struct extmnttab *mp)
 {
 	int ret;
 	struct stat64 st;
@@ -127,11 +127,7 @@ getextmntent(const char *path, struct extmnttab *entry, struct stat64 *statbuf)
 	}
 
 
-#ifdef HAVE_SETMNTENT
-	if ((fp = setmntent(MNTTAB, "re")) == NULL) {
-#else
 	if ((fp = fopen(MNTTAB, "re")) == NULL) {
-#endif
 		(void) fprintf(stderr, "cannot open %s\n", MNTTAB);
 		return (-1);
 	}
@@ -141,7 +137,7 @@ getextmntent(const char *path, struct extmnttab *entry, struct stat64 *statbuf)
 	 */
 
 	match = 0;
-	while (getextmntent_impl(fp, entry, sizeof (*entry)) == 0) {
+	while (getextmntent_impl(fp, entry) == 0) {
 		if (makedev(entry->mnt_major, entry->mnt_minor) ==
 		    statbuf->st_dev) {
 			match = 1;
