@@ -1,4 +1,3 @@
-/* BEGIN CSTYLED */
 /*
 ** $Id: lstrlib.c,v 1.178.1.1 2013/04/12 18:48:47 roberto Exp $
 ** Standard library for string operations and pattern-matching
@@ -501,7 +500,7 @@ static const char *match (MatchState *ms, const char *s, const char *p) {
             }
             case '+':  /* 1 or more repetitions */
               s++;  /* 1 match already done */
-              fallthrough;
+              zfs_fallthrough;
             case '*':  /* 0 or more repetitions */
               s = max_expand(ms, s, p, ep);
               break;
@@ -1037,4 +1036,3 @@ LUAMOD_API int luaopen_string (lua_State *L) {
 EXPORT_SYMBOL(luaopen_string);
 
 #endif
-/* END CSTYLED */
