@@ -1,4 +1,5 @@
 #!/bin/sh
+# shellcheck disable=SC2154
 #
 # CDDL HEADER START
 #
@@ -8,7 +9,7 @@
 # with the License.
 #
 # You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
-# or http://www.opensolaris.org/os/licensing.
+# or https://opensource.org/licenses/CDDL-1.0.
 # See the License for the specific language governing permissions
 # and limitations under the License.
 #
@@ -25,13 +26,8 @@
 # Copyright 2020 OmniOS Community Edition (OmniOSce) Association.
 #
 
-BASE_DIR=$(dirname "$0")
-SCRIPT_COMMON=common.sh
-if [ -f "${BASE_DIR}/${SCRIPT_COMMON}" ]; then
-. "${BASE_DIR}/${SCRIPT_COMMON}"
-else
-echo "Missing helper script ${SCRIPT_COMMON}" && exit 1
-fi
+SCRIPT_COMMON=${SCRIPT_COMMON:-${0%/*}/common.sh}
+. "${SCRIPT_COMMON}" || exit
 
 PROG=zfs-tests.sh
 VERBOSE="no"
@@ -52,7 +48,7 @@ TAGS=""
 ITERATIONS=1
 ZFS_DBGMSG="$STF_SUITE/callbacks/zfs_dbgmsg.ksh"
 ZFS_DMESG="$STF_SUITE/callbacks/zfs_dmesg.ksh"
-UNAME=$(uname -s)
+UNAME=$(uname)
 RERUN=""
 KMEMLEAK=""
 
@@ -99,7 +95,7 @@ cleanup_linux_loopback() {
 	for TEST_LOOPBACK in ${LOOPBACKS}; do
 		LOOP_DEV="${TEST_LOOPBACK##*/}"
 		DM_DEV=$(sudo "${DMSETUP}" ls 2>/dev/null | \
-		    grep "${LOOP_DEV}" | cut -f1)
+		    awk -v l="${LOOP_DEV}" '$0 ~ l {print $1}')
 
 		if [ -n "$DM_DEV" ]; then
 			sudo "${DMSETUP}" remove "${DM_DEV}" ||
@@ -132,9 +128,8 @@ cleanup() {
 		fi
 	fi
 
-	for TEST_FILE in ${FILES}; do
-		rm -f "${TEST_FILE}" >/dev/null 2>&1
-	done
+	# shellcheck disable=SC2086
+	rm -f ${FILES} >/dev/null 2>&1
 
 	if [ "$STF_PATH_REMOVE" = "yes" ] && [ -d "$STF_PATH" ]; then
 		rm -Rf "$STF_PATH"
@@ -149,28 +144,30 @@ trap cleanup EXIT
 # be dangerous and should only be used in a dedicated test environment.
 #
 cleanup_all() {
-	TEST_POOLS=$(sudo "$ZPOOL" list -H -o name | grep testpool)
+	TEST_POOLS=$(ASAN_OPTIONS=detect_leaks=false "$ZPOOL" list -Ho name | grep testpool)
 	if [ "$UNAME" = "FreeBSD" ] ; then
 		TEST_LOOPBACKS=$(sudo "${LOSETUP}" -l)
 	else
-		TEST_LOOPBACKS=$(sudo "${LOSETUP}" -a|grep file-vdev|cut -f1 -d:)
+		TEST_LOOPBACKS=$("${LOSETUP}" -a | awk -F: '/file-vdev/ {print $1}')
 	fi
-	TEST_FILES=$(ls /var/tmp/file-vdev* 2>/dev/null)
+	TEST_FILES=$(ls "${FILEDIR}"/file-vdev* /var/tmp/file-vdev* 2>/dev/null)
 
 	msg
 	msg "--- Cleanup ---"
-	msg "Removing pool(s):     $(echo "${TEST_POOLS}" | tr '\n' ' ')"
+	# shellcheck disable=2116,2086
+	msg "Removing pool(s):     $(echo ${TEST_POOLS})"
 	for TEST_POOL in $TEST_POOLS; do
-		sudo "$ZPOOL" destroy "${TEST_POOL}"
+		sudo env ASAN_OPTIONS=detect_leaks=false "$ZPOOL" destroy "${TEST_POOL}"
 	done
 
 	if [ "$UNAME" != "FreeBSD" ] ; then
-		msg "Removing dm(s):       $(sudo "${DMSETUP}" ls |
+		msg "Removing all dm(s):   $(sudo "${DMSETUP}" ls |
 		    grep loop | tr '\n' ' ')"
 		sudo "${DMSETUP}" remove_all
 	fi
 
-	msg "Removing loopback(s): $(echo "${TEST_LOOPBACKS}" | tr '\n' ' ')"
+	# shellcheck disable=2116,2086
+	msg "Removing loopback(s): $(echo ${TEST_LOOPBACKS})"
 	for TEST_LOOPBACK in $TEST_LOOPBACKS; do
 		if [ "$UNAME" = "FreeBSD" ] ; then
 			sudo "${LOSETUP}" -d -u "${TEST_LOOPBACK}"
@@ -179,10 +176,10 @@ cleanup_all() {
 		fi
 	done
 
-	msg "Removing files(s):    $(echo "${TEST_FILES}" | tr '\n' ' ')"
-	for TEST_FILE in $TEST_FILES; do
-		sudo rm -f "${TEST_FILE}"
-	done
+	# shellcheck disable=2116,2086
+	msg "Removing files(s):    $(echo ${TEST_FILES})"
+	# shellcheck disable=2086
+	sudo rm -f ${TEST_FILES}
 }
 
 #
@@ -196,19 +193,18 @@ cleanup_all() {
 #
 find_runfile() {
 	NAME=$1
-	RESULT=""
 
 	if [ -f "$RUNFILE_DIR/$NAME" ]; then
-		RESULT="$RUNFILE_DIR/$NAME"
+		echo "$RUNFILE_DIR/$NAME"
 	elif [ -f "$RUNFILE_DIR/$NAME.run" ]; then
-		RESULT="$RUNFILE_DIR/$NAME.run"
+		echo "$RUNFILE_DIR/$NAME.run"
 	elif [ -f "$NAME" ]; then
-		RESULT="$NAME"
+		echo "$NAME"
 	elif [ -f "$NAME.run" ]; then
-		RESULT="$NAME.run"
+		echo "$NAME.run"
+	else
+		return 1
 	fi
-
-	echo "$RESULT"
 }
 
 #
@@ -252,7 +248,7 @@ constrain_path() {
 	SYSTEM_DIRS="$SYSTEM_DIRS /usr/bin /usr/sbin /bin /sbin $LIBEXEC_DIR"
 
 	if [ "$INTREE" = "yes" ]; then
-		# Constrained path set to ./zfs/bin/
+		# Constrained path set to $(top_builddir)/tests/zfs-tests/bin
 		STF_PATH="$BIN_DIR"
 		STF_PATH_REMOVE="no"
 		STF_MISSING_BIN=""
@@ -262,14 +258,10 @@ constrain_path() {
 		fi
 
 		# Special case links for standard zfs utilities
-		DIRS="$(find "$CMD_DIR" -type d \( ! -name .deps -a \
-		    ! -name .libs \) -print | tr '\n' ' ')"
-		create_links "$DIRS" "$ZFS_FILES"
+		create_links "$CMD_DIR" "$ZFS_FILES"
 
 		# Special case links for zfs test suite utilities
-		DIRS="$(find "$STF_SUITE" -type d \( ! -name .deps -a \
-		    ! -name .libs \) -print | tr '\n' ' ')"
-		create_links "$DIRS" "$ZFSTEST_FILES"
+		create_links "$CMD_DIR/tests/zfs-tests/cmd" "$ZFSTEST_FILES"
 	else
 		# Constrained path set to /var/tmp/constrained_path.*
 		SYSTEMDIR=${SYSTEMDIR:-/var/tmp/constrained_path.XXXXXX}
@@ -296,14 +288,11 @@ constrain_path() {
 	create_links "$SYSTEM_DIRS" "$SYSTEM_FILES"
 
 	# Exceptions
-	ln -fs "$STF_PATH/awk" "$STF_PATH/nawk"
 	if [ "$UNAME" = "Linux" ] ; then
 		ln -fs /sbin/fsck.ext4 "$STF_PATH/fsck"
 		ln -fs /sbin/mkfs.ext4 "$STF_PATH/newfs"
 		ln -fs "$STF_PATH/gzip" "$STF_PATH/compress"
 		ln -fs "$STF_PATH/gunzip" "$STF_PATH/uncompress"
-		ln -fs "$STF_PATH/exportfs" "$STF_PATH/share"
-		ln -fs "$STF_PATH/exportfs" "$STF_PATH/unshare"
 	elif [ "$UNAME" = "FreeBSD" ] ; then
 		ln -fs /usr/local/bin/ksh93 "$STF_PATH/ksh"
 	fi
@@ -334,7 +323,7 @@ OPTIONS:
 	-m          Enable kmemleak reporting (Linux only)
 	-n NFSFILE  Use the nfsfile to determine the NFS configuration
 	-I NUM      Number of iterations
-	-d DIR      Use DIR for files and loopback devices
+	-d DIR      Use world-writable DIR for files and loopback devices
 	-s SIZE     Use vdevs of SIZE (default: 4G)
 	-r RUNFILES Run tests in RUNFILES (default: ${DEFAULT_RUNFILES})
 	-t PATH     Run single test at PATH relative to test suite
@@ -342,7 +331,7 @@ OPTIONS:
 	-u USER     Run single test as USER (default: root)
 
 EXAMPLES:
-# Run the default (linux) suite of tests and output the configuration used.
+# Run the default ($(echo "${DEFAULT_RUNFILES}" | sed 's/\.run//')) suite of tests and output the configuration used.
 $0 -v
 
 # Run a smaller suite of tests designed to run more quickly.
@@ -352,7 +341,7 @@ $0 -r linux-fast
 $0 -t tests/functional/cli_root/zfs_bookmark/zfs_bookmark_cliargs.ksh
 
 # Cleanup a previous run of the test suite prior to testing, run the
-# default (linux) suite of tests and perform no cleanup on exit.
+# default ($(echo "${DEFAULT_RUNFILES}" | sed 's/\.run//')) suite of tests and perform no cleanup on exit.
 $0 -x
 
 EOF
@@ -432,6 +421,8 @@ while getopts 'hvqxkKfScRmn:d:s:r:?t:T:u:I:' OPTION; do
 		usage
 		exit
 		;;
+	*)
+		;;
 	esac
 done
 
@@ -446,13 +437,9 @@ if [ -n "$SINGLETEST" ]; then
 	fi
 	RUNFILE_DIR="/var/tmp"
 	RUNFILES="zfs-tests.$$.run"
-	SINGLEQUIET="False"
+	[ -n "$QUIET" ] && SINGLEQUIET="True" || SINGLEQUIET="False"
 
-	if [ -n "$QUIET" ]; then
-		SINGLEQUIET="True"
-	fi
-
-	cat >$RUNFILE_DIR/$RUNFILES << EOF
+	cat >"${RUNFILE_DIR}/${RUNFILES}" << EOF
 [DEFAULT]
 pre =
 quiet = $SINGLEQUIET
@@ -463,20 +450,15 @@ post_user = root
 post =
 outputdir = /var/tmp/test_results
 EOF
-	SINGLETESTDIR=$(dirname "$SINGLETEST")
-	SINGLETESTFILE=$(basename "$SINGLETEST")
-	SETUPSCRIPT=
-	CLEANUPSCRIPT=
+	SINGLETESTDIR="${SINGLETEST%/*}"
 
-	if [ -f "$STF_SUITE/$SINGLETESTDIR/setup.ksh" ]; then
-		SETUPSCRIPT="setup"
-	fi
+	SETUPDIR="$SINGLETESTDIR"
+	[ "${SETUPDIR#/}" = "$SETUPDIR" ] && SETUPDIR="$STF_SUITE/$SINGLETESTDIR"
+	[ -x "$SETUPDIR/setup.ksh"   ] && SETUPSCRIPT="setup"     || SETUPSCRIPT=
+	[ -x "$SETUPDIR/cleanup.ksh" ] && CLEANUPSCRIPT="cleanup" || CLEANUPSCRIPT=
 
-	if [ -f "$STF_SUITE/$SINGLETESTDIR/cleanup.ksh" ]; then
-		CLEANUPSCRIPT="cleanup"
-	fi
-
-	cat >>$RUNFILE_DIR/$RUNFILES << EOF
+	SINGLETESTFILE="${SINGLETEST##*/}"
+	cat >>"${RUNFILE_DIR}/${RUNFILES}" << EOF
 
 [$SINGLETESTDIR]
 tests = ['$SINGLETESTFILE']
@@ -499,8 +481,8 @@ IFS=,
 for RUNFILE in $RUNFILES; do
 	if [ -n "$RUNFILE" ]; then
 		SAVED_RUNFILE="$RUNFILE"
-		RUNFILE=$(find_runfile "$RUNFILE")
-		[ -z "$RUNFILE" ] && fail "Cannot find runfile: $SAVED_RUNFILE"
+		RUNFILE=$(find_runfile "$RUNFILE") ||
+			fail "Cannot find runfile: $SAVED_RUNFILE"
 		R="$R,$RUNFILE"
 	fi
 
@@ -520,7 +502,7 @@ if [ "$(id -u)" = "0" ]; then
 	fail "This script must not be run as root."
 fi
 
-if [ "$(sudo whoami)" != "root" ]; then
+if [ "$(sudo id -un)" != "root" ]; then
 	fail "Passwordless sudo access required."
 fi
 
@@ -557,16 +539,14 @@ fi
 
 #
 # By default preserve any existing pools
-# NOTE: Since 'zpool list' outputs a newline-delimited list convert $KEEP from
-# space-delimited to newline-delimited.
 #
 if [ -z "${KEEP}" ]; then
-	KEEP="$(sudo "$ZPOOL" list -H -o name)"
+	KEEP="$(ASAN_OPTIONS=detect_leaks=false "$ZPOOL" list -Ho name | tr -s '[:space:]' ' ')"
 	if [ -z "${KEEP}" ]; then
 		KEEP="rpool"
 	fi
 else
-	KEEP="$(echo "$KEEP" | tr '[:blank:]' '\n')"
+	KEEP="$(echo "$KEEP" | tr -s '[:space:]' ' ')"
 fi
 
 #
@@ -578,11 +558,7 @@ fi
 #
 # See libzfs/libzfs_config.c for more information.
 #
-if [ "$UNAME" = "FreeBSD" ] ; then
-	__ZFS_POOL_EXCLUDE="$(echo "$KEEP" | tr -s '\n' ' ')"
-else
-	__ZFS_POOL_EXCLUDE="$(echo "$KEEP" | sed ':a;N;s/\n/ /g;ba')"
-fi
+__ZFS_POOL_EXCLUDE="$KEEP"
 
 . "$STF_SUITE/include/default.cfg"
 
@@ -622,8 +598,7 @@ if [ -z "${DISKS}" ]; then
 				DISKS="$DISKS $MDDEVICE"
 				LOOPBACKS="$LOOPBACKS $MDDEVICE"
 			else
-				TEST_LOOPBACK=$(sudo "${LOSETUP}" -f)
-				sudo "${LOSETUP}" "${TEST_LOOPBACK}" "${TEST_FILE}" ||
+				TEST_LOOPBACK=$(sudo "${LOSETUP}" --show -f "${TEST_FILE}") ||
 				    fail "Failed: ${TEST_FILE} -> ${TEST_LOOPBACK}"
 				BASELOOPBACK="${TEST_LOOPBACK##*/}"
 				DISKS="$DISKS $BASELOOPBACK"
@@ -649,7 +624,7 @@ fi
 #
 # Disable SELinux until the ZFS Test Suite has been updated accordingly.
 #
-if [ -x "$STF_PATH/setenforce" ]; then
+if command -v setenforce >/dev/null; then
 	sudo setenforce permissive >/dev/null 2>&1
 fi
 
@@ -657,8 +632,8 @@ fi
 # Enable internal ZFS debug log and clear it.
 #
 if [ -e /sys/module/zfs/parameters/zfs_dbgmsg_enable ]; then
-	sudo /bin/sh -c "echo 1 >/sys/module/zfs/parameters/zfs_dbgmsg_enable"
-	sudo /bin/sh -c "echo 0 >/proc/spl/kstat/zfs/dbgmsg"
+	sudo sh -c "echo 1 >/sys/module/zfs/parameters/zfs_dbgmsg_enable"
+	sudo sh -c "echo 0 >/proc/spl/kstat/zfs/dbgmsg"
 fi
 
 msg
@@ -688,7 +663,6 @@ export FILEDIR
 export KEEP
 export __ZFS_POOL_EXCLUDE
 export TESTFAIL_CALLBACKS
-export PATH=$STF_PATH
 
 mktemp_file() {
 	if [ "$UNAME" = "FreeBSD" ]; then
@@ -712,13 +686,18 @@ msg "${TEST_RUNNER}" \
     "-T \"${TAGS}\"" \
     "-i \"${STF_SUITE}\"" \
     "-I \"${ITERATIONS}\""
-${TEST_RUNNER} ${QUIET:+-q} ${KMEMLEAK:+-m} \
+{ PATH=$STF_PATH \
+    ${TEST_RUNNER} \
+    ${QUIET:+-q} \
+    ${KMEMLEAK:+-m} \
     ${KMSG:+-K} \
     -c "${RUNFILES}" \
     -T "${TAGS}" \
     -i "${STF_SUITE}" \
     -I "${ITERATIONS}" \
-    2>&1 | tee "$RESULTS_FILE"
+    2>&1; echo $? >"$REPORT_FILE"; } | tee "$RESULTS_FILE"
+read -r RUNRESULT <"$REPORT_FILE"
+
 #
 # Analyze the results.
 #
@@ -733,13 +712,17 @@ if [ "$RESULT" -eq "2" ] && [ -n "$RERUN" ]; then
 	for test_name in $MAYBES; do
 		grep "$test_name " "$TEMP_RESULTS_FILE" >>"$TEST_LIST"
 	done
-	${TEST_RUNNER} ${QUIET:+-q} ${KMEMLEAK:+-m} \
+	{ PATH=$STF_PATH \
+	    ${TEST_RUNNER} \
+	        ${QUIET:+-q} \
+	        ${KMEMLEAK:+-m} \
 	    -c "${RUNFILES}" \
 	    -T "${TAGS}" \
 	    -i "${STF_SUITE}" \
 	    -I "${ITERATIONS}" \
 	    -l "${TEST_LIST}" \
-	    2>&1 | tee "$RESULTS_FILE"
+	    2>&1; echo $? >"$REPORT_FILE"; } | tee "$RESULTS_FILE"
+	read -r RUNRESULT <"$REPORT_FILE"
 	#
 	# Analyze the results.
 	#
@@ -755,10 +738,10 @@ if [ -d "$RESULTS_DIR" ]; then
 	cat "$RESULTS_FILE" "$REPORT_FILE" >"$RESULTS_DIR/results"
 fi
 
-rm -f "$RESULTS_FILE" "$REPORT_FILE"
+rm -f "$RESULTS_FILE" "$REPORT_FILE" "$TEST_LIST" "$TEMP_RESULTS_FILE"
 
 if [ -n "$SINGLETEST" ]; then
 	rm -f "$RUNFILES" >/dev/null 2>&1
 fi
 
-exit ${RESULT}
+[ "$RUNRESULT" -gt 3 ] && exit "$RUNRESULT" || exit "$RESULT"
